package ch.hepia.openTranportAPI;

import org.json.JSONObject;

import java.sql.Time;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Set;

public final class Stop {
    private Location station;
    private Time arrival;
    private Time departure;
    private int delay;
    private int plateform;
    private Prognosis prognosis;

    /**
     * Constructor of the Stop class
     * @param station Location where the Stop is
     * @param arrival time of arrival at this Stop
     * @param departure time of departure from this Stop
     * @param delay duration time of unpredictated delay
     * @param plateform where the Stop will arrive to and depart from
     * @param prognosis complementary informations (See also {@link Prognosis})
     */
    private Stop(Location station, Time arrival, Time departure, int delay, int plateform, Prognosis prognosis){
        this.station = station;
        this.arrival = arrival;
        this.departure = departure;
        this.delay = delay;
        this.plateform = plateform;
        this.prognosis = prognosis;
    }

    /**
     * Get the Location of the Stop
     * @return station
     */
    public Location getLocation(){
        return station;
    }

    /**
     * Get the departure time of the transport from the Stop
     * @return departure
     */
    public Time getDepartureTime(){
        return departure;
    }

    /**
     * Get the Arrival time of the transport to the Stop
     * @return arrival
     */
    public Time getArrivalTime(){
        return arrival;
    }

    /**
     * Builder of Stop objects
     */
    public static class StopBuilder{
        private JSONObject datas;

        /**
         * Constructor of the class StopBuilder
         * Stock Datas from a JSON object given
         * @param datas contains datas to construct a Stop object
         */
        public StopBuilder(JSONObject datas){
            this.datas = datas;
        }

        /**
         * Build a Stop object from the datas obtained
         * @return a new Stop object
         * @throws ParseException
         */
        public Stop build() throws ParseException {
            //getting the first key to know the type of the Location
            String[] keys = (String[]) datas.keySet().toArray();
            String type = keys[0];
            JSONObject locationJSON = new JSONObject(datas.get(type));
            Location.LocationBuilder location = new Location.LocationBuilder(locationJSON, type);

            DateFormat formatter = new SimpleDateFormat("yyyy-MM-dddkk:mm:ss");

            String arrivalString = datas.getString("arrival");
            Time arrival = new Time(formatter.parse(arrivalString).getTime());
            String depatureString = datas.getString("departure");
            Time departure = new Time(formatter.parse(depatureString).getTime());

            int delay = datas.getInt("delay");
            int plateform = datas.getInt("plateform");

            JSONObject prognosisJSON = new JSONObject(datas.get("prognosis"));
            Prognosis.PrognosisBuilder prognosis = new Prognosis.PrognosisBuilder(prognosisJSON);

            return new Stop(location.build(), arrival, departure, delay, plateform, prognosis.build());
        }
    }
}
